#ifndef	CRI_INCL_CRI_ERROR_H
#define	CRI_INCL_CRI_ERROR_H
/****************************************************************************
 *
 * CRI Middleware SDK
 *
 * Copyright (c) 2006 CRI Middleware Co., Ltd.
 *
 * Library  : CRI Error
 * Module   : 
 * File     : cri_error.h
 *
 ****************************************************************************/
/*!
 * \file cri_error.h
 */
/*JP
 * \addtogroup CRI_ERROR
 * G[nhOp֐Q
 * @{
 */
/*EN
 * \addtogroup CRI_ERROR
 * Functions for error handling
 * @{
 */

/**************************************************************************** 
 *      Include file														* 
 ****************************************************************************/
#include "cri_le_xpt.h"

/**************************************************************************** 
 *		MACRO CONSTANT														* 
 ****************************************************************************/

/* o[Wԍ */
/*	Version number */
#define CRIERROR_VER_NAME				"CRI Error"
#define CRIERROR_VER_NUM				"1.4.5"
#define CRIERROR_VER_OPTION

/*JP
 * \brief G[R[h
 */
/*EN
 * \brief Error codes
 */
typedef enum {
	CRIERR_OK = 0,							/*JP< I */
											/*EN< Succeeded */
	CRIERR_NG = -1,							/*JP< G[ */
											/*EN< Error occurred */
	CRIERR_INVALID_PARAMETER = -2,			/*JP< s */
											/*EN< Invalid argument */
	CRIERR_FAILED_TO_ALLOCATE_MEMORY = -3,	/*JP< ̊mۂɎs */
											/*EN< Failed to allocate memory */
	CRIERR_UNSAFE_FUNCTION_CALL = -4,		/*JP< XbhZ[t֐̕s */
											/*EN< Parallel execution of thread-unsafe function */
	CRIERR_FUNCTION_NOT_IMPLEMENTED = -5,	/*JP< ֐̎s */
											/*EN< Function not implemented */
	CRIERR_LIBRARY_NOT_INITIALIZED = -6,	/*JP< Cu */
											/*EN< Library not initialized */
	/* enum be 4bytes */
	CRIERR_ENUM_BE_SINT32 = 0x7FFFFFFF
} CriError;

/*JP
 * \brief G[ʒmx
 */
/*EN
 * \brief Error notification level
 */
typedef enum {
	CRIERR_NOTIFY_ALL = 0,		/*JP< SẴG[ʒm */
								/*EN< Notify all errors */
	CRIERR_NOTIFY_FATAL = 1,	/*JP< G[̂ݒʒmix͖j */
								/*EN< Notify error (Disregards warning) */
	/* enum be 4bytes */
	CRIERR_NOTIFY_ENUM_BE_SINT32 = 0x7FFFFFFF
} CriErrorNotificationLevel;

/* G[x */
/* Error level */
typedef enum {
	CRIERR_LEVEL_ERROR = 0,
	CRIERR_LEVEL_WARNING = 1,
	/* enum be 4bytes */
	CRIERR_LEVEL_ENUM_BE_SINT32 = 0x7FFFFFFF
} CriErrorLevel;

/**************************************************************************** 
 *		PROCESS MACRO														* 
 ****************************************************************************/

#if defined(CRIERROR_SIMPLIFY)
	/* G[̒ʒm */
	/* Notification of Error */
	#define criErr_NotifyError(id, msg)							criErr_NotifyErrorSimple()
	#define criErr_NotifyError1(id, msg, p1)					criErr_NotifyErrorSimple()
	#define criErr_NotifyError2(id, msg, p1, p2)				criErr_NotifyErrorSimple()
	#define criErr_NotifyErrorPrmArray(id, msg, p1, p2, pa)		criErr_NotifyErrorSimple()
	#define criErr_NotifyErrorGeneric(_error_id, _error_no)		criErr_NotifyErrorSimple()
	/* x̒ʒm */
	/* Notification of Warning */
	#define criErr_NotifyWarning(id, msg)						criErr_NotifyWarningSimple()
	#define criErr_NotifyWarning1(id, msg, p1)					criErr_NotifyWarningSimple()
	#define criErr_NotifyWarning2(id, msg, p1, p2)				criErr_NotifyWarningSimple()
	#define criErr_NotifyWarningPrmArray(id, msg, p1, p2, pa)	criErr_NotifyWarningSimple()
	#define criErr_NotifyWarningGeneric(_error_id, _error_no)	criErr_NotifyWarningSimple()
#else
	/* G[̒ʒm */
	/* Notification of Error */
	#define criErr_NotifyError(id, msg)							criErr_Notify(CRIERR_LEVEL_ERROR, id ":" msg)
	#define criErr_NotifyError1(id, msg, p1)					criErr_Notify1(CRIERR_LEVEL_ERROR, (id ":" msg), (CriUintPtr)(p1))
	#define criErr_NotifyError2(id, msg, p1, p2)				criErr_Notify2(CRIERR_LEVEL_ERROR, (id ":" msg), (CriUintPtr)(p1), (CriUintPtr)(p2))
	#define criErr_NotifyErrorPrmArray(id, msg, p1, p2, pa)		criErr_NotifyPrmArray(CRIERR_LEVEL_ERROR, (id ":" msg), (CriUintPtr)(p1), (CriUintPtr)(p2), (CriUintPtr *)(pa))
	#define criErr_NotifyErrorGeneric(_error_id, _error_no)		criErr_NotifyGeneric(CRIERR_LEVEL_ERROR, _error_id, _error_no)
	/* x̒ʒm */
	/* Notification of Warning */
	#define criErr_NotifyWarning(id, msg)						criErr_Notify(CRIERR_LEVEL_WARNING, id ":" msg)
	#define criErr_NotifyWarning1(id, msg, p1)					criErr_Notify1(CRIERR_LEVEL_WARNING, (id ":" msg), (CriUintPtr)(p1))
	#define criErr_NotifyWarning2(id, msg, p1, p2)				criErr_Notify2(CRIERR_LEVEL_WARNING, (id ":" msg), (CriUintPtr)(p1), (CriUintPtr)(p2))
	#define criErr_NotifyWarningPrmArray(id, msg, p1, p2, pa)	criErr_NotifyPrmArray(CRIERR_LEVEL_WARNING, (id ":" msg), (CriUintPtr)(p1), (CriUintPtr)(p2), (CriUintPtr *)(pa))
	#define criErr_NotifyWarningGeneric(_error_id, _error_no)	criErr_NotifyGeneric(CRIERR_LEVEL_WARNING, _error_id, _error_no)
#endif

/**************************************************************************** 
 *      Data type declaration												* 
 ****************************************************************************/

/*	Error Callback Function type	*/
typedef void (CRIAPI *CriErrCbFunc)(const CriChar8 *errid, CriUint32 p1, CriUint32 p2, CriUint32 *parray);

/****************************************************************************
 *		ϐ̐錾															*
 *      Variable Declaration												*
 ****************************************************************************/
/* G[o͏ȗpϐ */
/* default argument of CriError */
#ifdef __cplusplus
namespace criErr {
	extern CriError ErrorContainer;
}
#endif	/* __cplusplus */

/****************************************************************************
 *		֐̐錾															*
 *      Function Declaration												*
 ****************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*JP
 * \brief		G[ID񂩂G[bZ[W֕ϊ
 * \ingroup 	CRI_ERROR
 * \param[in]	errid	G[ID
 * \return		G[bZ[W
 * \par :
 * G[ID񂩂ڍׂȃG[bZ[W֕ϊ܂B<br>
 * \attention
 * ̊֐͋dl̊֐łB<br>
 *  ::criErr_ConvertIdToMessage ֐gpĂB
 * \sa criErr_ConvertIdToMessage
 */
/*EN
 * \brief Convert error ID to error message
 * \ingroup 	CRI_ERROR
 * \param[in]	errid	error ID
 * \return		error message
 * \par Explanation:
 * This function converts error ID to detailed error message.<br>
 * \attention
 * This function is an obsolete one.<br>
 * Please use the ::criErr_ConvertIdToMessage function instead.
 * \sa criErr_ConvertIdToMessage
 */
const CriChar8* CRIAPI criErr_ConvertIdToMsg(const CriChar8 *errid);

/*JP
 * \brief		G[ID񂩂G[bZ[W֕ϊ
 * \ingroup 	CRI_ERROR
 * \param[in]	errid	G[ID
 * \param[in]	p1		⑫1
 * \param[in]	p2		⑫2
 * \return		G[bZ[W
 * \par :
 * G[ID񂩂ڍׂȃG[bZ[W֕ϊ܂B<br>
 * \par p:
 * \code
 * void user_error_callback_func(const CriChar8 *errid, CriUint32 p1, CriUint32 p2, CriUint32 *parray);
 * {
 * 	const CriChar8 *errmsg;
 * 	errmsg = criErr_ConvertIdToMessage(errid, p1, p2);
 * 	printf("%s\n", errmsg);
 * }
 * \endcode
 */
/*EN
 * \brief Convert error ID to error message
 * \ingroup 	CRI_ERROR
 * \param[in]	errid	error ID
 * \param[in]	p1		supplementary information 1
 * \param[in]	p2		supplementary information 2
 * \return		error message
 * \par Explanation:
 * This function converts error ID to detailed error message.<br>
 * \par Example:
 * \code
 * void user_error_callback_func(const CriChar8 *errid, CriUint32 p1, CriUint32 p2, CriUint32 *parray);
 * {
 * 	const CriChar8 *errmsg;
 * 	errmsg = criErr_ConvertIdToMessage(errid, p1, p2);
 * 	printf("%s\n", errmsg);
 * }
 * \endcode
 */
const CriChar8* CRIAPI criErr_ConvertIdToMessage(const CriChar8 *errid, CriUint32 p1, CriUint32 p2);

/*JP
 * \brief		G[R[obN֐̓o^
 * \ingroup 	CRI_ERROR
 * \param[in]	cbf	G[R[obN֐
 * \return		Ȃ
 * \par :
 * G[R[obN֐o^܂B<br>
 * o^ꂽ֐́ACRI~hEGACuŃG[ƂɌĂяo܂B<br>
 * \par p:
 * \code
 * void user_error_callback_func(const CriChar8 *errid, CriUint32 p1, CriUint32 p2, CriUint32 *parray);
 * {
 * 	const CriChar8 *errmsg;
 * 	errmsg = criErr_ConvertIdToMessage(errid, p1, p2);
 * 	printf("%s\n", errmsg);
 * }
 * 
 * void main(int ac, char *av[])
 * {
 * 		F
 * 	// G[R[obN̓o^
 * 	criErr_SetCallback(user_error_callback_func);
 * 		F
 * }
 * \endcode
 * \attention
 * ɓo^łG[R[obN֐͂PłB<br>
 * o^ɍēx{֐Ăяoꍇ݂͌̓o^㏑܂B<br>
 * <br>
 * G[R[obN̏㏑ꍇA㏑AꂽoɌxԂ܂B<br>
 * AvP[V̐ȏ̗ƂāAӐ}IɃG[R[obN֐̍ւsP[Xꍇɂ́A
 * UucriErr_SetCallback(NULL);vsăR[obN̓o^A
 * ̌ɉ߂ăG[R[obN֐̓o^sĂB
 */
/*EN
 * \brief		Register error callback function
 * \ingroup 	CRI_ERROR
 * \param[in]	cbf	error callback function
 * \return		NONE
 * \par Explanation:
 * This function registers an error callback function.<br>
 * The registered function is called if an error occurs within the CRI middleware library.
 * \par Example:
 * \code
 * void user_error_callback_func(const CriChar8 *errid, CriUint32 p1, CriUint32 p2, CriUint32 *parray);
 * {
 * 	const CriChar8 *errmsg;
 * 	errmsg = criErr_ConvertIdToMessage(errid, p1, p2);
 * 	printf("%s\n", errmsg);
 * }
 * 
 * void main(int ac, char *av[])
 * {
 * 		:
 * 	// Registers the error callback function
 * 	criErr_SetCallback(user_error_callback_func);
 * 		:
 * }
 * \endcode
 */
void CRIAPI criErr_SetCallback(CriErrCbFunc cbf);

/*JP
 *  \brief		G[ʒmx̕ύX
 *	\ingroup 	CRI_ERROR
 *	\param[in]	level	G[ʒmx
 *	\return		Ȃ
 *	\par :
 *	G[R[obNɒʒmG[̃xύX܂B
 */
/*EN
 * \brief Change error notification level
 *	\ingroup 	CRI_ERROR
 *	\param[in]	level	error notification level
 *	\return		NONE
 *	\par Explanation:
 *	This function changes the level of error information that is notified to the error callback.
 */
void CRIAPI criErr_SetErrorNotificationLevel(CriErrorNotificationLevel level);

/*JP
 *  \brief		G[񐔂̎擾
 *	\ingroup 	CRI_ERROR
 *	\param[in]	level	G[x
 *	\return		G[
 *	\par :
 *	G[񐔂擾܂B
 */
/*EN
 * \brief Retrieve error count
 *	\ingroup 	CRI_ERROR
 *	\param[in]	level	error level
 *	\return		number of errors occured
 *	\par Explanation:
 *	This function retrieves the number of errors occured.
 */
CriUint32 CRIAPI criErr_GetErrorCount(CriErrorLevel level);

/*JP
 *  \brief		G[񐔂̃Zbg
 *	\ingroup 	CRI_ERROR
 *	\param[in]	level	G[x
 *	\return		Ȃ
 *	\par :
 *	G[񐔂̃JE^0ɖ߂܂B
 */
/*EN
 * \brief Resets error count
 *	\ingroup 	CRI_ERROR
 *	\param[in]	level	error level
 *	\return		NONE
 *	\par Explanation:
 *	This function resets the counter for number of errors occured.
 */
void CRIAPI criErr_ResetErrorCount(CriErrorLevel level);

/* G[R[obN𔭐 (֐) */
/* Error notification (Internal functions) */

#ifdef __cplusplus
}
#endif /* __cplusplus */

/***************************************************************************
 *      o[WƂ̌݊p
 *      For compatibility with old versions
 ***************************************************************************/

/*EN
 * @}
 */
/*JP
 * @}
 */

#endif	//	CRI_INCL_CRI_ERROR_INTR_H

/* --- end of file --- */
